function [data, state] = checkinput(data, varargin);

% CHECKINPUT checks the input of the main FieldTrip functions, e.g. whether
% the type of data strucure corresponds with the required data. If neccessary
% and possible, this function will adjust the data structure to the input
% requirements (e.g. change dimord, average over trials, convert inside from
% index into logical).
%
% If the input data does NOT correspond to the requirements, this function
% is supposed to give a elaborate warning message and if applicable point
% the user to external documentation (link to website).
%
% Use as
%   [data, state] = checkinput(data, ...)
%
% Optional input arguments should be specified as key-value pairs and can 
% include
%   datatype            raw, wave, freq, timelock, comp, source, volume, dip
%   dimord              any combination of time, freq, chan, refchan, rpt, subj, chancmb, rpttap
%   megtype             ctf151, ctf275, ctf151_planar, ctf275_planar, neuromag122, neuromag306, bti148, magnetometer
%   precision           single, double
%   inside              logical, index
%   hastrials           yes, no
%   hascumtapcnt        yes  -> this is implied by datatype=freq
%   hasoffset           yes, only for raw data
%   hascrsspctrm        yes, can be computed from fourier
%   hasfourier          yes
%   ismeg               yes
%   iseeg               yes
%   csdrepresentation   square, cmb
%   covrepresentation   square, cmb
%
% For some options you can specify multiple values, e.g. 
%   [data] = checkinput(data, 'megtype', {'ctf151', 'ctf275'}), e.g. in megrealign 
%   [data] = checkinput(data, 'datatype', {'timelock', 'freq'}), e.g. in sourceanalysis

% Copyright (C) 2007, Robert Oostenveld
%
% $Log: checkinput.m,v $
% Revision 1.2  2007/02/27 13:35:01  roboos
% added some comments for ingnie to work on
%
% Revision 1.1  2007/02/27 09:24:11  roboos
% initial version that only serves as placeholder for the documentation and some example code
%

% in case of an error this function could use dbstack for more detailled
% user feedback
% 
% this function should replace/encapsulate
%   fixdimord
%   fixinside
%   fixprecision
%   fixvolume
%   data2raw
%   raw2data
%   grid2transform
%   transform2grid
%   fourier2crsspctrm
%   freq2cumtapcnt
%   sensortype
%   time2offset
%   offset2time
%
% other potential uses for this function:
%   time -> offset in freqanalysis
%   average over trials
%   csd as matrix

% get the optional input arguments
datatype  = keyval('datatype', varargin);
dimord    = keyval('dimord', varargin);
hastrials = keyval('hastrials', varargin);
inside    = keyval('inside', varargin);
% ...

% define a list of error strings
% ...

% define a list of strings that point to reference documentation
refdatarepr    = 'See the FieldTrip documentation on data representations in Matlab at http://tinyurl.com/4h3ife.';
refdataform    = 'See the FieldTrip documentation on data and file formats at http://tinyurl.com/yev4zg.';
refdatamegtype = 'See the FieldTrip documentation on supported MEG systems at http://tinyurl.com/3w4h43.';
% ...

% http://www2.ru.nl/fcdonders/fieldtrip/doku.php?id=fieldtrip:documentation:frequently_asked_questions#why_is_there_a_rim_around_the_brain_for_which_the_source_reconstruction_is_not_computed
% http://tinyurl.com/37jp5d

if ~isempty(datatype)

% this should be something like
if iscell(datatype)
  i = 1;
  datatype_ok = 0;
  while ~datatype_ok && i<length(datatype)
    datatype_ok = check_datatype(data, datatype{i})
    i = i+1;
  end
else  
  datatype_ok = check_datatype(data, datatype)
end

if datatype = {freq timelock} && ~ok
  error(ik verwacht freq of timelock data hierzo) % verder geen details in errmsg
elseif datatype = freq && ~ok
  error(ik verwacht freq data hierzo, je probleem is errmsg)
elseif datatype = timelock && ~ok
  error(ik verwacht timelock data hierzo,  je probleem is errmsg)
else
  error(ik verwacht andere data hierzo)
end

end % datatype

if ~isempty(dimord)
end % datatype

if ~isempty(hastrials)
end % datatype

if ~isempty(inside)
end % datatype



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% helper function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [errflag, errmsg] = check_datatype(data, datatype);
  errflag = 0;
  errmsg  = [];
  if strcmp(datatype, 'raw')
    if ~errflag && ~isfield(data, 'trial') || isa(data.trial, 'cell')
      errflag = 1;
      errmsg  = 'This function expects raw data as input, but the input data does not have time axes. Time axes should be represented in a cell-array.';
    end
    if ~errflag && ~isfield(data, 'time') || isa(data.time, 'cell')
      errflag = 2;
      errmsg  = 'This function expects raw data as input, but the input data does not have time axes. Time axes should be represented in a cell-array.';
    end
  end  
  ok = (errmsg==0);
